/***********************************************************************/
/*                                                                     */
/*       Programs written by Xin Zhang                                 */
/*       Courant Institute of Mathematical Science                     */
/*       Department of Computer Science                                */
/*       New York University                                           */
/*                                                                     */
/*       Programs modified by Yang Song                                */
/*       Department of Computer Science                                */
/*       New Jersey Institute of Technology                            */
/*       University Heights, Newark, NJ 07102, USA                     */
/*       RA in the group of Dr. Jason T. L. Wang (NJIT)                */
/*                                                                     */
/*	 Permission to use, copy, modify, and distribute this    	       */
/*	 software and its documentation for any purpose and without        */
/*	 fee is hereby granted, provided that this copyright	           */
/*	 notice appears in all copies.   Programmer(s) makes no	           */
/*	 representations about the suitability of this		               */
/*	 software for any purpose.  It is provided "as is" without         */
/*	 express or implied warranty.				                       */
/*                                                                     */
/***********************************************************************/


#ifndef SATFINDER_H
#define SATFINDER_H

#include <vector>
#include <set>
#include <string>
#include "SATNode.h"


using namespace std;

class SATFinder
{
public:
	SATFinder();
	virtual ~SATFinder();

	void searchSAT(char* thresFile, char* sampleFile, char* satFile);
	void setSearchParam(int expandSetSize, int numFinalNodeToStop);
	void clean();

	long _totalRunTime;

private:
	void stateSpaceSearch();
	double getTheoCost(SATNode* satNode);

	int loadThresholds(const char* thresFile);
	int loadSampleData(const char* sampleFile);
	void sampleStat();

	void insertToFrontier(SATNode* node);
	void removeFromFrontier(SATNode* node);

private:
	// thresholds
	vector<pair<int, double> > _thresholds;
	int		_maxWinSize;

	// sample data
	long	_numSampleData;
	double*	_sampleData;
	double	_mean;
	double  _stddev;
	double* _ap;

	// search
	set<string>			_visitedNodes;
	vector<SATNode*>	_frontier;
	vector<int>			_numExpandedByOverlapWinSize;
	vector<vector<SATNode*> >	_frontierByOverlapWinSize;

	SATNode*	_best10SAT[10];

	int		_maxFrontierSize;
	int		_numFinalNodeToStop;

};

#endif
